#!/bin/bash
#==============================================================================
# Copyright and license info is available in the LICENSE file included with
# the Helix Management System (HMS), and also available online:
# https://swarm.workshop.perforce.com/projects/perforce_software-hms/view/main/LICENSE
set -u

# Generate an SDP instance-specific variant of the generic
# P4Broker config file, suitable for use with HMS New Job
# (NJ) Package.  Display to standard output.

# Usage: In these examples, replace 'N' with SDP instance name.
# cd /p4/common/config
# /p4/common/bin/gen_nj_broker_cfg.sh N > p4_N.broker.cfg.ToBeReviewed
#
# After reviewing, remove the .ToBeReviewed suffix. Alternately, blend the
# generated content in an existing broker configuration file.

export SDP_INSTANCE=${1:-${SDP_INSTANCE:-Unset}}

if [[ "$SDP_INSTANCE" == Unset ]]; then 
   echo "Instance parameter not supplied." 
   echo "You must supply the SDP instance as a parameter to this script." 
   exit 1 
fi 

source /p4/common/bin/p4_vars "$SDP_INSTANCE"
declare BrokerLogLevel=1
declare BrokerConfigTemplate=p4_N.broker.cfg.t
declare HelixSwarmUser=swarm
declare NJVersionFile=Version
declare NJVersion=

if [[ ! -r "$BrokerConfigTemplate" ]]; then
   echo "Error: Could not find broker config template: [$PWD/$BrokerConfigTemplate]."
   exit 1
fi

if [[ ! -r "$BrokerConfigTemplate" ]]; then
   echo "Error: Could not find NJ Version file [$PWD/$NJVersionFile]."
   exit 1
fi

NJVersion=$(cat $NJVersionFile)

sed -e "s|__P4PORT__|$P4PORT|g" \
   -e "s|__P4USER__|$P4USER|g" \
   -e "s|__P4BROKERPORT__|$P4BROKERPORT|g" \
   -e "s|__P4BINDIR__|${P4BIN%/*}|g" \
   -e "s|__BROKER_LOG_LEVEL__|$BrokerLogLevel|g" \
   -e "s|__LOGS__|$LOGS|g" \
   -e "s|__MAILTO__|$MAILTO|g" \
   -e "s|__SWARM_USER__|$HelixSwarmUser|g" \
   -e "s|__NJ_VERSION__|$NJVersion|g" \
   "$BrokerConfigTemplate"
